import { useEffect, useState } from 'react'
import './App.css';
import Checkerboard from './views/Checkerboard'
import RoomList from './views/RoomList'
import Login from './views/Login';
import RevMsgContext from './provider/RevMsgContext'
import GlobalSocketContext from './provider/GlobalSocketContext'
import RevRoomContext from './provider/RevRoomContext'
import { ROOM, HALL } from './Constant'
function App() {
  const [pageState, updatePageState] = useState(0)
  const [roomList, updateRoomList] = useState("")
  const [roomData, updateRoomData] = useState("")
  const [socket, setSocket] = useState(null)
  const [userName, setUsername] = useState("")
  useEffect(()=>{
    if ( userName === "" ) return;
    const socket = new WebSocket(`ws://127.0.0.1:8080/game/${userName}`) 
    socket.onopen = ()=>{
      socket.send(JSON.stringify( {instruct: HALL} ))
    }
    socket.onerror = ( error )=>{
      console.log( error )
    }
    socket.onmessage = (msg)=>{
      const revMsg = JSON.parse(msg.data)
      switch ( revMsg.instruct ) {
        case HALL: updateRoomList(revMsg); return updatePageState(1)
        case ROOM: updateRoomData(revMsg); return updatePageState(2)
        default: return updatePageState(0)
      }
      
    }
    socket.onclose = (e)=>{
      console.log(e)
    }
    setSocket(socket)
  }, [userName])
  
  return (
    
      <GlobalSocketContext.Provider value={socket}>
        
        {
          (()=>{
            if ( pageState === 0 ) {
              return <Login setUsername={setUsername} />
            } else if ( pageState === 1 ) {
              return(
                <RevMsgContext.Provider value={roomList}>
                  <RoomList />
                </RevMsgContext.Provider>
              )
            } else if ( pageState === 2 ) {
              return (
                  <RevRoomContext.Provider value={roomData} >
                    <Checkerboard />
                  </RevRoomContext.Provider>
              )
            }
          })()
        }
      </GlobalSocketContext.Provider>
  );
}

export default App;
